@@ -3,7 +3,7 @@ package Couchbase::Bucket;
 use strict;
 use warnings;
 
-use JSON;
+use JSON::MaybeXS;
 use URI;
 use Storable;
 
@@ -13,12 +13,10 @@ use Couchbase::Document;
 use Couchbase::Settings;
 use Couchbase::OpContext;
 use Couchbase::View::Handle;
-use Couchbase::View::HandleInfo;
-use Couchbase::View::Handle::RawIterator;
-use Couchbase::View::Handle::ViewIterator;
-use Couchbase::View::Handle::Slurpee;
+use Couchbase::HTTPDocument;
+use Couchbase::N1QL::Handle;
 
-my $_JSON = JSON->new()->allow_nonref;
+my $_JSON = JSON::MaybeXS->new()->allow_nonref;
 sub _js_encode { $_JSON->encode($_[0]) }
 sub _js_decode { $_JSON->decode($_[0]) }
 
@@ -150,17 +148,33 @@ sub fetch {
     return $doc;
 }
 
-# Returns a 'raw' request handle
 sub _htraw {
-    my $self = $_[0];
-    return $self->_new_viewhandle(\%Couchbase::View::Handle::RawIterator::);
+    my ($self,$method,$path,$options) = @_;
+    $options ||= {};
+    $method = uc($method);
+
+    my $methmap = {
+        'GET' => LCB_HTTP_METHOD_GET,
+        'POST' => LCB_HTTP_METHOD_POST,
+        'PUT' => LCB_HTTP_METHOD_PUT,
+        'DELETE' => LCB_HTTP_METHOD_DELETE
+    };
+
+    my $methnum = $methmap->{$method};
+    if (!defined($methnum)) {
+        die("Unknown method $method");
+    }
+
+    $options->{method} = $methnum;
+    my $htd = Couchbase::HTTPDocument->new($path);
+    $self->_http($htd, $options);
+    return $htd;
 }
 
 # Gets a design document
 sub design_get {
     my ($self,$path) = @_;
-    my $handle = $self->_new_viewhandle(\%Couchbase::View::Handle::Slurpee::);
-    my $design = $handle->slurp_jsonized("GET", "_design/" . $path, "");
+    return $self->_htraw('GET', '_design/'.$path);
 }
 
 # saves a design document
@@ -170,57 +184,40 @@ sub design_put {
         $path = $design->{_id};
         $design = encode_json($design);
     }
-    my $handle = $self->_new_viewhandle(\%Couchbase::View::Handle::Slurpee::);
-    return $handle->slurp_jsonized("PUT", $path, $design);
-}
-
-sub _process_viewpath_common {
-    my ($orig,%options) = @_;
-    my %qparams;
-    if (%options) {
-        # TODO: pop any other known parameters?
-        %qparams = (%qparams,%options);
+    if (!$path) {
+        die("Cannot determine path for design document");
     }
 
-    if (ref $orig ne 'ARRAY') {
-        if (!$orig) {
-            die("Path cannot be empty");
-        }
-        $orig = [($orig =~ m,([^/]+)/(.*),)]
-    }
-
-    unless ($orig->[0] && $orig->[1]) {
-        die("Path cannot be empty");
-    }
-
-    # Assume this is an array of [ design, view ]
-    $orig = sprintf("_design/%s/_view/%s", @$orig);
-
-    if (%qparams) {
-        $orig = URI->new($orig);
-        $orig->query_form(\%qparams);
-    }
-
-    return $orig . "";
-}
-
-# slurp an entire resultset of views
-sub view_slurp {
-    my ($self,$viewpath,%options) = @_;
-    my $handle = $self->_new_viewhandle(\%Couchbase::View::Handle::Slurpee::);
-    $viewpath = _process_viewpath_common($viewpath,%options);
-    return $handle->slurp("GET", $viewpath, "");
+    return $self->_htraw('PUT', $path, {
+        body => $design,
+        content_type => 'application/json'
+    });
 }
 
 sub view_iterator {
     my ($self,$viewpath,%options) = @_;
-    my $handle;
+    my $iter = Couchbase::View::Handle->new($self, $viewpath, %options);
+    return $iter;
+}
 
-    $viewpath = _process_viewpath_common($viewpath, %options);
-    $handle = $self->_new_viewhandle(\%Couchbase::View::Handle::ViewIterator::);
-    $handle->_perl_initialize();
-    $handle->prepare("GET", $viewpath, "");
-    return $handle;
+sub view_slurp {
+    my $self = shift;
+    my $iter = $self->view_iterator(@_);
+    $iter->slurp();
+    return $iter;
+}
+
+sub query_iterator {
+    my ($self, $query, $params, $options) = @_;
+    my $iter = Couchbase::N1QL::Handle->new($self, $query, $params, $options);
+    return $iter;
+}
+
+sub query_slurp {
+    my $self = shift;
+    my $iter = $self->query_iterator(@_);
+    $iter->slurp();
+    return $iter;
 }
 
 sub bucket {
@@ -725,15 +722,130 @@ The C<options> passed can be C<persist_to> and C<replicate_to>. See the
 L<"Durability Requirements"> section for information.
 
 
+=head2 N1QL QUERIES (EXPERIMENTAL)
+
+
+N1QL queries are available as an experimental feature of the client
+library.
+
+The N1QL API exposes two functions, both of which function
+similarly to their view counterparts.
+
+At the time of writing, the server does not include N1QL as an integrated
+feature (because it is still experimental). This means it must be downloaded
+as a standalone package
+(see L<http://docs.couchbase.com/developer/n1ql-dp4/n1ql-intro.html>). Once
+downloaded and configured, the C<_host> option should be passed to the
+query function (as detailed below).
+
+N1QL functions return a L<Couchbase::N1QL::Handle> object, which functions
+similarly to L<Couchbase::View::Handle> (internally, they share a lot of
+code).
+
+
+=head3 query_slurp("query", $queryargs, $queryopts)
+
+Issue an N1QL query. This will send the query to the server (encoding
+any parameters as needed).
+
+    my $rv = $cb->query_slurp(
+        # Query string
+        'SELECT *, META().id FROM travel WHERE travel.country = $country ',
+
+        # Placeholder values
+        { country => "Ecuador", },
+
+        # Query options
+        { _host => "localhost:8093" }
+    );
+
+    foreach my $row (@{$rv->rows}) {
+        # do something with decoded JSON
+    }
+
+
+The C<queryargs> parameter can either be a hashref of named placeholders
+(omiting of course, the leading C<$> which is handled internally), or it can
+be an arrayref of positional placeholders (if your query uses positional
+placeholders).
+
+The C<queryopts> is a set of other modifiers for the query. Most of these
+are sent to the server. One special parameter is the C<_host> parameter, which
+points to a standalone instance of the N1QL Developer Preview installation;
+a temporary necesity for pre-release versions. Using of the C<_host> paramter
+will be removed once Couchbase Server is available (in release or pre-release)
+with an integrated N1QL process.
+
+
+=head3 query_iterator("query", $queryargs, $queryopts)
+
+This function is to C<query_slurp> as C<view_iterator> is to C<view_slurp>.
+In short, this allows an iterator over the rows, only fetching data from
+the network as needed. This is more efficient (but a bit less simple to
+use) than C<query_slurp>
+
+    my $rv = $cb->query_iterator("select * from default");
+    while ((my $row = $rv->next)) {
+        # do something with row.
+    }
+
+
 =head2 VIEW (MAPREDUCE) QUERIES
 
 
+View methods come in two flavors. One is an iterator which incrementally
+fetches data from the network, while the other loads the entire data and
+then returns. For small queries (i.e. those which do not return many
+results), which API you use is a matter of personal preference. For larger
+resultsets, however, it often becomes a necessity to not load the entire
+dataset into RAM.
+
+Both the C<view_slurp> and C<view_iterator> return L<Couchbase::View::Handle>
+objects. This has been changed from previous versions which returned a
+C<Couchbase::View::HandleInfo> object (Though the APIs remain the same).
+
+
 =head3 view_slurp("design/view", %options)
 
 Queries and returns the results of a view. The first argument may be provided
 either as a string of C<"$design/$view"> or as a two-element array reference
 containing the design and view respectively.
 
+The C<%options> are options passed verbatim to the view engine. Some options
+however are intercepted by the client, and modify how the view is queried.
+
+=over
+
+=item C<spatial>
+
+Indicate that the queried view is a geospatial view. This is required since the
+formatting of the internal URI is slightly different.
+
+=item C<include_docs>
+
+Indicate that the relevant documents should be fetched for each view. The
+following forms are equivalent.
+
+    # fetching directly:
+    my $iter = $bkt->view_iterator(['design', 'view']);
+    while ((my $row = $iter->next)) {
+        my $doc = Couchbase::Document->new($row->id);
+        $bkt->get($doc);
+    }
+
+    # using include_docs
+    my $iter = $bkt->view_iterator(['design', 'view'], include_docs => 1);
+    while ((my $row = $iter->next)) {
+        my $doc = $row->doc;
+    }
+
+Using C<include_docs> is significantly more efficient than fetching the rows
+manually as it allows the library to issue gets in bulk for each raw chunk
+of view results received - and also allows the library to "lazily" fetch
+documents while other rows are being received.
+
+=back
+
 The returned object contains various status information about the query. The
 rows themselves may be found inside the C<rows> accessor:
 
@@ -743,7 +855,7 @@ rows themselves may be found inside the C<rows> accessor:
     }
 
 
-This method returns an instance of L<Couchbase::View::HandleInfo> which may be used
+This method returns an instance of L<Couchbase::View::Handle> which may be used
 to inspect for error messages. The object is in fact a subclass of
 L<Couchbase::Document> with an additional C<errinfo> method to provide more
 details about the operation.
@@ -757,6 +869,8 @@ details about the operation.
         }
     }
 
+As of version 2.0.3, this method is implemented as a wrapper atop C<view_iterator>
+
 
 =head3 view_iterator("design/view", %options)
 
@@ -770,12 +884,9 @@ query to return a large amount of results:
         printf("Got row for key %s with document id %s\n", $row->key, $row->id);
     }
 
-Note that unlike the C<view_slurp> method, this does I<not> return a
-C<HandleInfo> object, but rather a L<Couchbase::View::Handle::ViewIterator>. The
-actual C<HandleInfo> object can be obtained using the C<info> method of the
-iterator. Note that the contents of the C<HandleInfo> object are only
-considered valid once the iterator has been through at least I<one> iteration;
-thus:
+
+Note that the contents of the C<Handle> object are only considered valid once
+the iterator has been through at least I<one> iteration; thus:
 
 B<Incorrect>, because it requests the C<info> object before iteration has
 started
